@@ -1,7 +1,10 @@
-/*
- * mlt_frame.c -- interface for all frame classes
- * Copyright (C) 2003-2004 Ushodaya Enterprises Limited
- * Author: Charles Yates <charles.yates@pandora.be>
+/**
+ * \file mlt_frame.c
+ * \brief interface for all frame classes
+ * \see mlt_frame_s
+ *
+ * Copyright (C) 2003-2009 Ushodaya Enterprises Limited
+ * \author Charles Yates <charles.yates@pandora.be>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +21,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "config.h"
 #include "mlt_frame.h"
 #include "mlt_producer.h"
 #include "mlt_factory.h"
@@ -32,13 +34,15 @@
 /** Constructor for a frame.
 */
 
-mlt_frame mlt_frame_init( )
+mlt_frame mlt_frame_init( mlt_service service )
 {
 	// Allocate a frame
 	mlt_frame this = calloc( sizeof( struct mlt_frame_s ), 1 );
 
 	if ( this != NULL )
 	{
+		mlt_profile profile = mlt_service_profile( service );
+
 		// Initialise the properties
 		mlt_properties properties = &this->parent;
 		mlt_properties_init( properties, this );
@@ -46,10 +50,10 @@ mlt_frame mlt_frame_init( )
 		// Set default properties on the frame
 		mlt_properties_set_position( properties, "_position", 0.0 );
 		mlt_properties_set_data( properties, "image", NULL, 0, NULL, NULL );
-		mlt_properties_set_int( properties, "width", mlt_profile_get()->width );
-		mlt_properties_set_int( properties, "height", mlt_profile_get()->height );
-		mlt_properties_set_int( properties, "normalised_width", mlt_profile_get()->width );
-		mlt_properties_set_int( properties, "normalised_height", mlt_profile_get()->height );
+		mlt_properties_set_int( properties, "width", profile? profile->width : 720 );
+		mlt_properties_set_int( properties, "height", profile? profile->height : 576 );
+		mlt_properties_set_int( properties, "normalised_width", profile? profile->width : 720 );
+		mlt_properties_set_int( properties, "normalised_height", profile? profile->height : 576 );
 		mlt_properties_set_double( properties, "aspect_ratio", mlt_profile_sar( NULL ) );
 		mlt_properties_set_data( properties, "audio", NULL, 0, NULL, NULL );
 		mlt_properties_set_data( properties, "alpha", NULL, 0, NULL, NULL );
@@ -215,12 +219,12 @@ mlt_deque mlt_frame_service_stack( mlt_frame this )
 	of a composite where the b frame completely obscures the a frame).
 
 	The image must be writable and the destructor for the image itself must be taken
-	care of on another frame and that frame cannot have a replace applied to it... 
+	care of on another frame and that frame cannot have a replace applied to it...
 	Further it assumes that no alpha mask is in use.
 
-	For these reasons, it can only be used in a specific situation - when you have 
+	For these reasons, it can only be used in a specific situation - when you have
 	multiple tracks each with their own transition and these transitions are applied
-	in a strictly reversed order (ie: highest numbered [lowest track] is processed 
+	in a strictly reversed order (ie: highest numbered [lowest track] is processed
 	first).
 
 	More reliable approach - the cases should be detected during the process phase
@@ -232,7 +236,7 @@ void mlt_frame_replace_image( mlt_frame this, uint8_t *image, mlt_image_format f
 	// Remove all items from the stack
 	while( mlt_deque_pop_back( this->stack_image ) ) ;
 
-	// Update the information 
+	// Update the information
 	mlt_properties_set_data( MLT_FRAME_PROPERTIES( this ), "image", image, 0, NULL, NULL );
 	mlt_properties_set_int( MLT_FRAME_PROPERTIES( this ), "width", width );
 	mlt_properties_set_int( MLT_FRAME_PROPERTIES( this ), "height", height );
@@ -445,24 +449,24 @@ unsigned char *mlt_frame_get_waveform( mlt_frame this, int w, int h )
 	mlt_audio_format format = mlt_audio_pcm;
 	int frequency = 32000; // lower frequency available?
 	int channels = 2;
-	double fps = mlt_properties_get_double( properties, "fps" );
+	double fps = mlt_profile_fps( NULL );
 	int samples = mlt_sample_calculator( fps, frequency, mlt_frame_get_position( this ) );
-	
+
 	// Get the pcm data
 	mlt_frame_get_audio( this, &pcm, &format, &frequency, &channels, &samples );
-	
+
 	// Make an 8-bit buffer large enough to hold rendering
 	int size = w * h;
 	unsigned char *bitmap = ( unsigned char* )mlt_pool_alloc( size );
 	if ( bitmap != NULL )
 		memset( bitmap, 0, size );
 	mlt_properties_set_data( properties, "waveform", bitmap, size, ( mlt_destructor )mlt_pool_release, NULL );
-	
+
 	// Render vertical lines
 	int16_t *ubound = pcm + samples * channels;
 	int skip = samples / w - 1;
 	int i, j, k;
-	
+
 	// Iterate sample stream and along x coordinate
 	for ( i = 0; i < w && pcm < ubound; i++ )
 	{
@@ -471,7 +475,7 @@ unsigned char *mlt_frame_get_waveform( mlt_frame this, int w, int h )
 		{
 			// Determine sample's magnitude from 2s complement;
 			int pcm_magnitude = *pcm < 0 ? ~(*pcm) + 1 : *pcm;
-			// The height of a line is the ratio of the magnitude multiplied by 
+			// The height of a line is the ratio of the magnitude multiplied by
 			// half the vertical resolution
 			int height = ( int )( ( double )( pcm_magnitude ) / 32768 * h / 2 );
 			// Determine the starting y coordinate - left channel above center,
@@ -479,11 +483,11 @@ unsigned char *mlt_frame_get_waveform( mlt_frame this, int w, int h )
 			int displacement = ( h / 2 ) - ( 1 - j ) * height;
 			// Position buffer pointer using y coordinate, stride, and x coordinate
 			unsigned char *p = &bitmap[ i + displacement * w ];
-			
+
 			// Draw vertical line
 			for ( k = 0; k < height; k++ )
 				p[ w * k ] = 0xFF;
-			
+
 			pcm++;
 		}
 		pcm += skip * channels;
@@ -521,7 +525,7 @@ int mlt_convert_yuv422_to_rgb24a( uint8_t *yuv, uint8_t *rgba, unsigned int tota
 	int yy, uu, vv;
       	int r,g,b;
 	total /= 2;
-	while (total--) 
+	while (total--)
 	{
 		yy = yuv[0];
 		uu = yuv[1];
@@ -1063,7 +1067,7 @@ uint8_t *mlt_frame_rescale_yuv422( mlt_frame this, int owidth, int oheight )
     	{
         	// Start at the beginning of the line
         	out_ptr = out_line;
-	
+
         	// Pointer to the middle of the input line
         	in_line = in_middle + ( dy >> 16 ) * istride;
 
@@ -1219,7 +1223,7 @@ int mlt_frame_combine_audio( mlt_frame this, mlt_frame that, int16_t **buffer, m
  	double B = exp(-2.0 * M_PI * Fc);
 	double A = 1.0 - B;
 	double v;
-	
+
 	for ( i = 0; i < *samples; i++ )
 	{
 		for ( j = 0; j < *channels; j++ )
