@@ -1,7 +1,10 @@
-/*
- * mlt_multitrack.c -- multitrack service class
- * Copyright (C) 2003-2004 Ushodaya Enterprises Limited
- * Author: Charles Yates <charles.yates@pandora.be>
+/**
+ * \file mlt_multitrack.c
+ * \brief multitrack service class
+ * \see mlt_multitrack_s
+ *
+ * Copyright (C) 2003-2009 Ushodaya Enterprises Limited
+ * \author Charles Yates <charles.yates@pandora.be>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,8 +21,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "config.h"
-
 #include "mlt_multitrack.h"
 #include "mlt_playlist.h"
 #include "mlt_frame.h"
@@ -27,13 +28,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-/** Forward reference.
-*/
+/* Forward reference. */
 
-static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int index );
+static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int index );
 
-/** Constructor.
-*/
+/** Construct and initialize a new multitrack.
+ *
+ * Sets the resource property to "<multitrack>".
+ *
+ * \public \memberof mlt_multitrack_s
+ * \return a new multitrack
+ */
 
 mlt_multitrack mlt_multitrack_init( )
 {
@@ -61,12 +66,17 @@ mlt_multitrack mlt_multitrack_init( )
 			this = NULL;
 		}
 	}
-	
+
 	return this;
 }
 
 /** Get the producer associated to this multitrack.
-*/
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \return the producer object
+ * \see MLT_MULTITRACK_PRODUCER
+ */
 
 mlt_producer mlt_multitrack_producer( mlt_multitrack this )
 {
@@ -74,7 +84,12 @@ mlt_producer mlt_multitrack_producer( mlt_multitrack this )
 }
 
 /** Get the service associated this multitrack.
-*/
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \return the service object
+ * \see MLT_MULTITRACK_SERVICE
+ */
 
 mlt_service mlt_multitrack_service( mlt_multitrack this )
 {
@@ -82,15 +97,23 @@ mlt_service mlt_multitrack_service( mlt_multitrack this )
 }
 
 /** Get the properties associated this multitrack.
-*/
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \return the multitrack's property list
+ * \see MLT_MULTITRACK_PROPERTIES
+ */
 
 mlt_properties mlt_multitrack_properties( mlt_multitrack this )
 {
 	return MLT_MULTITRACK_PROPERTIES( this );
 }
 
-/** Initialise position related information.
-*/
+/** Initialize position related information.
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ */
 
 void mlt_multitrack_refresh( mlt_multitrack this )
 {
@@ -102,9 +125,6 @@ void mlt_multitrack_refresh( mlt_multitrack this )
 	// We need to ensure that the multitrack reports the longest track as its length
 	mlt_position length = 0;
 
-	// We need to ensure that fps are the same on all services
-	double fps = 0;
-	
 	// Obtain stats on all connected services
 	for ( i = 0; i < this->count; i ++ )
 	{
@@ -118,25 +138,10 @@ void mlt_multitrack_refresh( mlt_multitrack this )
 			// If we have more than 1 track, we must be in continue mode
 			if ( this->count > 1 )
 				mlt_properties_set( MLT_PRODUCER_PROPERTIES( producer ), "eof", "continue" );
-			
+
 			// Determine the longest length
 			//if ( !mlt_properties_get_int( MLT_PRODUCER_PROPERTIES( producer ), "hide" ) )
 				length = mlt_producer_get_playtime( producer ) > length ? mlt_producer_get_playtime( producer ) : length;
-			
-			// Handle fps
-			if ( fps == 0 )
-			{
-				// This is the first producer, so it controls the fps
-				fps = mlt_producer_get_fps( producer );
-			}
-			else if ( fps != mlt_producer_get_fps( producer ) )
-			{
-				// Generate a warning for now - the following attempt to fix may fail
-				fprintf( stderr, "Warning: fps mismatch on track %d\n", i );
-
-				// It should be safe to impose fps on an image producer, but not necessarily safe for video
-				mlt_properties_set_double( MLT_PRODUCER_PROPERTIES( producer ), "fps", fps );
-			}
 		}
 	}
 
@@ -145,11 +150,14 @@ void mlt_multitrack_refresh( mlt_multitrack this )
 	mlt_properties_set_position( properties, "length", length );
 	mlt_events_unblock( properties, properties );
 	mlt_properties_set_position( properties, "out", length - 1 );
-	mlt_properties_set_double( properties, "fps", fps );
 }
 
 /** Listener for producers on the playlist.
-*/
+ *
+ * \private \memberof mlt_multitrack_s
+ * \param producer a producer
+ * \param this a multitrack
+ */
 
 static void mlt_multitrack_listener( mlt_producer producer, mlt_multitrack this )
 {
@@ -157,10 +165,16 @@ static void mlt_multitrack_listener( mlt_producer producer, mlt_multitrack this
 }
 
 /** Connect a producer to a given track.
-
-  	Note that any producer can be connected here, but see special case treatment
-	of playlist in clip point determination below.
-*/
+ *
+ * Note that any producer can be connected here, but see special case treatment
+ * of playlist in clip point determination below.
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \param producer the producer to connect to the multitrack producer
+ * \param track the 0-based index of the track on which to connect the multitrack
+ * \return true on error
+ */
 
 int mlt_multitrack_connect( mlt_multitrack this, mlt_producer producer, int track )
 {
@@ -191,15 +205,15 @@ int mlt_multitrack_connect( mlt_multitrack this, mlt_producer producer, int trac
 
 		// Assign the track in our list here
 		this->list[ track ]->producer = producer;
-		this->list[ track ]->event = mlt_events_listen( MLT_PRODUCER_PROPERTIES( producer ), this, 
+		this->list[ track ]->event = mlt_events_listen( MLT_PRODUCER_PROPERTIES( producer ), this,
 									 "producer-changed", ( mlt_listener )mlt_multitrack_listener );
 		mlt_properties_inc_ref( MLT_PRODUCER_PROPERTIES( producer ) );
 		mlt_event_inc_ref( this->list[ track ]->event );
-		
+
 		// Increment the track count if need be
 		if ( track >= this->count )
 			this->count = track + 1;
-			
+
 		// Refresh our stats
 		mlt_multitrack_refresh( this );
 	}
@@ -208,31 +222,57 @@ int mlt_multitrack_connect( mlt_multitrack this, mlt_producer producer, int trac
 }
 
 /** Get the number of tracks.
-*/
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \return the number of tracks
+ */
 
 int mlt_multitrack_count( mlt_multitrack this )
 {
-	return this->count;	
+	return this->count;
 }
 
 /** Get an individual track as a producer.
-*/
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \param track the 0-based index of the producer to get
+ * \return the producer or NULL if not valid
+ */
 
 mlt_producer mlt_multitrack_track( mlt_multitrack this, int track )
 {
 	mlt_producer producer = NULL;
-	
+
 	if ( this->list != NULL && track < this->count )
 		producer = this->list[ track ]->producer;
 
 	return producer;
 }
 
+/** Position comparison function for sorting.
+ *
+ * \private \memberof mlt_multitrack_s
+ * \param p1 a position
+ * \param p2 another position
+ * \return <0 if \p p1 is less than \p p2, 0 if equal, >0 if greater
+ */
+
 static int position_compare( const void *p1, const void *p2 )
 {
 	return *( mlt_position * )p1 - *( mlt_position * )p2;
 }
 
+/** Add a position to a set.
+ *
+ * \private \memberof mlt_multitrack_s
+ * \param array an array of positions (the set)
+ * \param size the current number of positions in the array (not the capacity of the array)
+ * \param position the position to add
+ * \return the new size of the array
+ */
+
 static int add_unique( mlt_position *array, int size, mlt_position position )
 {
 	int i = 0;
@@ -245,24 +285,32 @@ static int add_unique( mlt_position *array, int size, mlt_position position )
 }
 
 /** Determine the clip point.
-
-  	Special case here: a 'producer' has no concept of multiple clips - only the 
-	playlist and multitrack producers have clip functionality. Further to that a 
-	multitrack determines clip information from any connected tracks that happen 
-	to be playlists.
-
-	Additionally, it must locate clips in the correct order, for example, consider
-	the following track arrangement:
-
-	playlist1 |0.0     |b0.0      |0.1          |0.1         |0.2           |
-	playlist2 |b1.0  |1.0           |b1.1     |1.1             |
-
-	Note - b clips represent blanks. They are also reported as clip positions.
-
-	When extracting clip positions from these playlists, we should get a sequence of:
-
-	0.0, 1.0, b0.0, 0.1, b1.1, 1.1, 0.1, 0.2, [out of playlist2], [out of playlist1]
-*/
+ *
+ * <pre>
+ * Special case here: a 'producer' has no concept of multiple clips - only the
+ * playlist and multitrack producers have clip functionality. Further to that a
+ * multitrack determines clip information from any connected tracks that happen
+ * to be playlists.
+ *
+ * Additionally, it must locate clips in the correct order, for example, consider
+ * the following track arrangement:
+ *
+ * playlist1 |0.0     |b0.0      |0.1          |0.1         |0.2           |
+ * playlist2 |b1.0  |1.0           |b1.1     |1.1             |
+ *
+ * Note - b clips represent blanks. They are also reported as clip positions.
+ *
+ * When extracting clip positions from these playlists, we should get a sequence of:
+ *
+ * 0.0, 1.0, b0.0, 0.1, b1.1, 1.1, 0.1, 0.2, [out of playlist2], [out of playlist1]
+ * </pre>
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ * \param whence from where to extract
+ * \param index the 0-based index of which clip to extract
+ * \return the position of clip \p index relative to \p whence
+ */
 
 mlt_position mlt_multitrack_clip( mlt_multitrack this, mlt_whence whence, int index )
 {
@@ -316,7 +364,7 @@ mlt_position mlt_multitrack_clip( mlt_multitrack this, mlt_whence whence, int in
 
 		case mlt_whence_relative_current:
 			position = mlt_producer_position( MLT_MULTITRACK_PRODUCER( this ) );
-			for ( i = 0; i < count - 2; i ++ ) 
+			for ( i = 0; i < count - 2; i ++ )
 				if ( position >= map[ i ] && position < map[ i + 1 ] )
 					break;
 			index += i;
@@ -343,29 +391,37 @@ mlt_position mlt_multitrack_clip( mlt_multitrack this, mlt_whence whence, int in
 }
 
 /** Get frame method.
-
-  	Special case here: The multitrack must be used in a conjunction with a downstream
-	tractor-type service, ie:
-
-	Producer1 \
-	Producer2 - multitrack - { filters/transitions } - tractor - consumer
-	Producer3 /
-
-	The get_frame of a tractor pulls frames from it's connected service on all tracks and 
-	will terminate as soon as it receives a test card with a last_track property. The 
-	important case here is that the mulitrack does not move to the next frame until all
-	tracks have been pulled. 
-
-	Reasoning: In order to seek on a network such as above, the multitrack needs to ensure
-	that all producers are positioned on the same frame. It uses the 'last track' logic
-	to determine when to move to the next frame.
-
-	Flaw: if a transition is configured to read from a b-track which happens to trigger
-	the last frame logic (ie: it's configured incorrectly), then things are going to go
-	out of sync.
-
-	See playlist logic too.
-*/
+ *
+ * <pre>
+ * Special case here: The multitrack must be used in a conjunction with a downstream
+ * tractor-type service, ie:
+ *
+ * Producer1 \
+ * Producer2 - multitrack - { filters/transitions } - tractor - consumer
+ * Producer3 /
+ *
+ * The get_frame of a tractor pulls frames from it's connected service on all tracks and
+ * will terminate as soon as it receives a test card with a last_track property. The
+ * important case here is that the mulitrack does not move to the next frame until all
+ * tracks have been pulled.
+ *
+ * Reasoning: In order to seek on a network such as above, the multitrack needs to ensure
+ * that all producers are positioned on the same frame. It uses the 'last track' logic
+ * to determine when to move to the next frame.
+ *
+ * Flaw: if a transition is configured to read from a b-track which happens to trigger
+ * the last frame logic (ie: it's configured incorrectly), then things are going to go
+ * out of sync.
+ *
+ * See playlist logic too.
+ * </pre>
+ *
+ * \private \memberof mlt_multitrack_s
+ * \param parent the producer interface to a mulitrack
+ * \param[out] frame a frame by reference
+ * \param index the 0-based track index
+ * \return true if there was an error
+ */
 
 static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int index )
 {
@@ -405,7 +461,7 @@ static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int ind
 	else
 	{
 		// Generate a test frame
-		*frame = mlt_frame_init( );
+		*frame = mlt_frame_init( MLT_PRODUCER_SERVICE( parent ) );
 
 		// Update position on the frame we're creating
 		mlt_frame_set_position( *frame, mlt_producer_position( parent ) );
@@ -424,8 +480,11 @@ static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int ind
 	return 0;
 }
 
-/** Close this instance.
-*/
+/** Close this instance and free its resources.
+ *
+ * \public \memberof mlt_multitrack_s
+ * \param this a multitrack
+ */
 
 void mlt_multitrack_close( mlt_multitrack this )
 {
