@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "transition_affine.h"
+#include <framework/mlt_transition.h>
 #include <framework/mlt.h>
 
 #include <stdio.h>
@@ -510,7 +510,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 
 					if ( dx >= 0 && dx < b_width && dy >=0 && dy < b_height )
 					{
-						*pmask ++;
+						pmask ++;
 						dx -= dx & 1;
 						*p ++ = *( b_image + dy * b_stride + ( dx << 1 ) );
 						*p ++ = *( b_image + dy * b_stride + ( dx << 1 ) + ( ( x & 1 ) << 1 ) + 1 );
@@ -592,7 +592,7 @@ static mlt_frame transition_process( mlt_transition transition, mlt_frame a_fram
 /** Constructor for the filter.
 */
 
-mlt_transition transition_affine_init( char *arg )
+mlt_transition transition_affine_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg )
 {
 	mlt_transition transition = mlt_transition_new( );
 	if ( transition != NULL )
