@@ -26,6 +26,8 @@
 
 #include <framework/mlt.h>
 
+#include "config.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -36,12 +38,18 @@ struct producer_qimage_s
 	mlt_properties filenames;
 	int count;
 	int image_idx;
+	uint8_t *current_image;
+	uint8_t *current_alpha;
+	int current_width;
+	int current_height;	
 };
 
 typedef struct producer_qimage_s *producer_qimage;
 
 extern void refresh_qimage( mlt_frame, int width, int height );
+#ifdef USE_KDE
 extern void init_qimage();
+#endif
 
 #ifdef __cplusplus
 }
