@@ -19,28 +19,11 @@
  */
 
 #include <string.h>
+#include <framework/mlt.h>
 
-#include "consumer_valerie.h"
+extern mlt_consumer consumer_valerie_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 
-void *mlt_create_producer( char *id, void *arg )
+MLT_REPOSITORY
 {
-	return NULL;
+	MLT_REGISTER( consumer_type, "valerie", consumer_valerie_init );
 }
-
-void *mlt_create_filter( char *id, void *arg )
-{
-	return NULL;
-}
-
-void *mlt_create_transition( char *id, void *arg )
-{
-	return NULL;
-}
-
-void *mlt_create_consumer( char *id, void *arg )
-{
-	if ( !strcmp( id, "valerie" ) )
-		return consumer_valerie_init( arg );
-	return NULL;
-}
-
